#!/usr/bin/env node

// src/index.ts
import { McpServer as McpServer4 } from "@modelcontextprotocol/sdk/server/mcp.js";
import { StdioServerTransport } from "@modelcontextprotocol/sdk/server/stdio.js";

// src/resources/decks.ts
import { ResourceTemplate } from "@modelcontextprotocol/sdk/server/mcp.js";

// src/utils/ankiClient.ts
import { YankiConnect } from "yanki-connect";
var ankiClient = new YankiConnect({
  autoLaunch: true
});

// src/resources/decks.ts
function registerDeckResources(server2) {
  server2.resource("deck_names", "anki:///decks/names", async (uri) => {
    try {
      const deckNames = await ankiClient.deck.deckNames();
      return {
        contents: [
          {
            uri: uri.href,
            mimeType: "application/json",
            text: JSON.stringify(deckNames, null, 2)
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get deck names: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.resource("deck_names_and_ids", "anki:///decks/names-and-ids", async (uri) => {
    try {
      const deckNamesAndIds = await ankiClient.deck.deckNamesAndIds();
      return {
        contents: [
          {
            uri: uri.href,
            mimeType: "application/json",
            text: JSON.stringify(deckNamesAndIds, null, 2)
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get deck names and IDs: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.resource(
    "deck_config",
    new ResourceTemplate("anki:///decks/{deckName}/config", { list: void 0 }),
    async (uri, { deckName }) => {
      const deckNameString = Array.isArray(deckName) ? deckName[0] : deckName;
      if (!deckNameString) {
        throw new Error("Deck name is required");
      }
      try {
        const config = await ankiClient.deck.getDeckConfig({ deck: deckNameString });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(config, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get deck config for "${deckNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "deck_stats",
    new ResourceTemplate("anki:///decks/{deckNames}/stats", { list: void 0 }),
    async (uri, { deckNames }) => {
      const deckNamesString = Array.isArray(deckNames) ? deckNames[0] : deckNames;
      if (!deckNamesString) {
        throw new Error("Deck names are required");
      }
      const deckNamesArray = deckNamesString.split(",").map((name) => name.trim());
      if (deckNamesArray.length === 0) {
        throw new Error("At least one deck name is required");
      }
      try {
        const stats = await ankiClient.deck.getDeckStats({ decks: deckNamesArray });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(stats, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get deck stats for "${deckNamesArray.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "decks_by_cards",
    new ResourceTemplate("anki:///decks/by-cards/{cardIds}", { list: void 0 }),
    async (uri, { cardIds }) => {
      const cardIdsString = Array.isArray(cardIds) ? cardIds[0] : cardIds;
      if (!cardIdsString) {
        throw new Error("Card IDs are required");
      }
      const cardIdArray = cardIdsString.split(",").map((id) => {
        const num = parseInt(id.trim(), 10);
        if (isNaN(num)) {
          throw new Error(`Invalid card ID: ${id}`);
        }
        return num;
      });
      if (cardIdArray.length === 0) {
        throw new Error("At least one card ID is required");
      }
      try {
        const decks = await ankiClient.deck.getDecks({ cards: cardIdArray });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(decks, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get decks for cards "${cardIdArray.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/resources/cards.ts
import { ResourceTemplate as ResourceTemplate2 } from "@modelcontextprotocol/sdk/server/mcp.js";
function registerCardResources(server2) {
  server2.resource(
    "find_cards",
    new ResourceTemplate2("anki:///cards/search/{query}", { list: void 0 }),
    async (uri) => {
      try {
        const query = decodeURIComponent(uri.pathname.split("/").pop() || "");
        if (!query) {
          throw new Error("Query parameter is required");
        }
        const cardIds = await ankiClient.card.findCards({ query });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(cardIds, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to find cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_info",
    new ResourceTemplate2("anki:///cards/{cardIds}/info", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const cardsInfo = await ankiClient.card.cardsInfo({ cards: cardIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(cardsInfo, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get cards info: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_due_status",
    new ResourceTemplate2("anki:///cards/{cardIds}/due", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const areDue = await ankiClient.card.areDue({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          isDue: areDue[index]
        }));
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(result, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if cards are due: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_suspend_status",
    new ResourceTemplate2("anki:///cards/{cardIds}/suspended", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const areSuspended = await ankiClient.card.areSuspended({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          isSuspended: areSuspended[index]
        }));
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(result, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if cards are suspended: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_mod_time",
    new ResourceTemplate2("anki:///cards/{cardIds}/mod-time", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const modTimes = await ankiClient.card.cardsModTime({ cards: cardIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(modTimes, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get cards modification time: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_to_notes",
    new ResourceTemplate2("anki:///cards/{cardIds}/notes", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const noteIds = await ankiClient.card.cardsToNotes({ cards: cardIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(noteIds, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get note IDs from cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_ease_factors",
    new ResourceTemplate2("anki:///cards/{cardIds}/ease-factors", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const easeFactors = await ankiClient.card.getEaseFactors({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          easeFactor: easeFactors[index]
        }));
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(result, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get ease factors: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_intervals",
    new ResourceTemplate2("anki:///cards/{cardIds}/intervals", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const url = new URL(uri.href);
        const complete = url.searchParams.get("complete") === "true";
        const intervals = await ankiClient.card.getIntervals({ cards: cardIds, complete });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          intervals: intervals[index]
        }));
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(result, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get intervals: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/resources/notes.ts
import { ResourceTemplate as ResourceTemplate3 } from "@modelcontextprotocol/sdk/server/mcp.js";
function registerNoteResources(server2) {
  server2.resource("note_tags", "anki:///notes/tags", async (uri) => {
    try {
      const tags = await ankiClient.note.getTags();
      return {
        contents: [
          {
            uri: uri.href,
            mimeType: "application/json",
            text: JSON.stringify({
              tags: tags.sort(),
              count: tags.length,
              description: "All available tags in the Anki collection"
            }, null, 2)
          }
        ]
      };
    } catch (error) {
      throw new Error(`Failed to get tags: ${error instanceof Error ? error.message : String(error)}`);
    }
  });
  server2.resource(
    "notes_info",
    new ResourceTemplate3("anki:///notes/{noteIds}/info", { list: void 0 }),
    async (uri) => {
      try {
        const noteIdsParam = uri.pathname.split("/").slice(-2)[0];
        const noteIds = noteIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (noteIds.some((id) => isNaN(id))) {
          throw new Error("Invalid note IDs provided");
        }
        const notesInfo = await ankiClient.note.notesInfo({ notes: noteIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify({
                notes: notesInfo,
                count: notesInfo.length,
                description: "Detailed information about the requested notes"
              }, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(`Failed to get notes info: ${error instanceof Error ? error.message : String(error)}`);
      }
    }
  );
  server2.resource(
    "notes_search",
    new ResourceTemplate3("anki:///notes/search/{query}", { list: void 0 }),
    async (uri) => {
      try {
        const query = decodeURIComponent(uri.pathname.split("/").pop() || "");
        if (!query) {
          throw new Error("Search query is required");
        }
        const noteIds = await ankiClient.note.findNotes({ query });
        const limitedIds = noteIds.slice(0, 50);
        const notesInfo = limitedIds.length > 0 ? await ankiClient.note.notesInfo({ notes: limitedIds }) : [];
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify({
                query,
                totalFound: noteIds.length,
                displayedCount: limitedIds.length,
                noteIds,
                notes: notesInfo,
                description: `Search results for query: "${query}". Showing detailed info for first 50 notes.`
              }, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(`Failed to search notes: ${error instanceof Error ? error.message : String(error)}`);
      }
    }
  );
  server2.resource(
    "notes_by_tag",
    new ResourceTemplate3("anki:///notes/tag/{tag}", { list: void 0 }),
    async (uri) => {
      try {
        const tag = decodeURIComponent(uri.pathname.split("/").pop() || "");
        if (!tag) {
          throw new Error("Tag is required");
        }
        const query = `tag:${tag}`;
        const noteIds = await ankiClient.note.findNotes({ query });
        const limitedIds = noteIds.slice(0, 50);
        const notesInfo = limitedIds.length > 0 ? await ankiClient.note.notesInfo({ notes: limitedIds }) : [];
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify({
                tag,
                totalFound: noteIds.length,
                displayedCount: limitedIds.length,
                noteIds,
                notes: notesInfo,
                description: `Notes tagged with "${tag}". Showing detailed info for first 50 notes.`
              }, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(`Failed to get notes by tag: ${error instanceof Error ? error.message : String(error)}`);
      }
    }
  );
  server2.resource(
    "notes_recent",
    new ResourceTemplate3("anki:///notes/recent/{days}", { list: void 0 }),
    async (uri) => {
      try {
        const daysParam = uri.pathname.split("/").pop() || "7";
        const days = parseInt(daysParam, 10);
        if (isNaN(days) || days <= 0) {
          throw new Error("Days must be a positive number");
        }
        const query = `edited:${days}`;
        const noteIds = await ankiClient.note.findNotes({ query });
        const limitedIds = noteIds.slice(0, 50);
        const notesInfo = limitedIds.length > 0 ? await ankiClient.note.notesInfo({ notes: limitedIds }) : [];
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify({
                days,
                totalFound: noteIds.length,
                displayedCount: limitedIds.length,
                noteIds,
                notes: notesInfo,
                description: `Notes modified in the last ${days} days. Showing detailed info for first 50 notes.`
              }, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(`Failed to get recent notes: ${error instanceof Error ? error.message : String(error)}`);
      }
    }
  );
  server2.resource(
    "notes_mod_times",
    new ResourceTemplate3("anki:///notes/{noteIds}/mod-times", { list: void 0 }),
    async (uri) => {
      try {
        const noteIdsParam = uri.pathname.split("/").slice(-2)[0];
        const noteIds = noteIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (noteIds.some((id) => isNaN(id))) {
          throw new Error("Invalid note IDs provided");
        }
        const modTimes = await ankiClient.note.notesModTime({ notes: noteIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify({
                modificationTimes: modTimes,
                count: modTimes.length,
                description: "Modification timestamps for the requested notes"
              }, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(`Failed to get note modification times: ${error instanceof Error ? error.message : String(error)}`);
      }
    }
  );
}

// src/tools/decks.ts
import { z } from "zod";
function registerDeckTools(server2) {
  server2.tool(
    "create_deck",
    {
      deckName: z.string().describe("Name of the deck to create")
    },
    async ({ deckName }) => {
      try {
        const result = await ankiClient.deck.createDeck({ deck: deckName });
        return {
          content: [
            {
              type: "text",
              text: `Successfully created deck "${deckName}". Result: ${JSON.stringify(result)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to create deck "${deckName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "delete_decks",
    {
      deckNames: z.array(z.string()).describe("Array of deck names to delete"),
      deleteCards: z.boolean().default(true).describe("Whether to delete the cards in the decks as well")
    },
    async ({ deckNames, deleteCards }) => {
      try {
        await ankiClient.deck.deleteDecks({
          decks: deckNames,
          cardsToo: deleteCards
          // Type assertion needed due to yanki-connect's strict typing
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully deleted decks: ${deckNames.join(", ")}${deleteCards ? " (including cards)" : " (cards preserved)"}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to delete decks "${deckNames.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "change_deck",
    {
      cardIds: z.array(z.number()).describe("Array of card IDs to move"),
      targetDeck: z.string().describe("Name of the target deck to move cards to")
    },
    async ({ cardIds, targetDeck }) => {
      try {
        await ankiClient.deck.changeDeck({
          cards: cardIds,
          deck: targetDeck
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully moved ${cardIds.length} cards to deck "${targetDeck}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to move cards to deck "${targetDeck}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "clone_deck_config",
    {
      sourceConfigId: z.number().describe("ID of the deck configuration to clone from"),
      newConfigName: z.string().describe("Name for the new cloned configuration")
    },
    async ({ sourceConfigId, newConfigName }) => {
      try {
        const result = await ankiClient.deck.cloneDeckConfigId({
          cloneFrom: sourceConfigId,
          name: newConfigName
        });
        if (result === false) {
          throw new Error("Failed to clone deck configuration - operation returned false");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully cloned deck configuration. New config ID: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to clone deck configuration: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "remove_deck_config",
    {
      configId: z.number().describe("ID of the deck configuration to remove")
    },
    async ({ configId }) => {
      try {
        const result = await ankiClient.deck.removeDeckConfigId({ configId });
        if (!result) {
          throw new Error("Failed to remove deck configuration - operation returned false");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully removed deck configuration with ID: ${configId}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to remove deck configuration: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "save_deck_config",
    {
      config: z.object({
        id: z.number(),
        name: z.string(),
        autoplay: z.boolean(),
        dyn: z.boolean(),
        maxTaken: z.number(),
        mod: z.number(),
        replayq: z.boolean(),
        timer: z.number(),
        usn: z.number(),
        lapse: z.object({
          delays: z.array(z.number()),
          leechAction: z.number(),
          leechFails: z.number(),
          minInt: z.number(),
          mult: z.number()
        }),
        new: z.object({
          bury: z.boolean(),
          delays: z.array(z.number()),
          initialFactor: z.number(),
          ints: z.array(z.number()),
          order: z.number(),
          perDay: z.number(),
          separate: z.boolean()
        }),
        rev: z.object({
          bury: z.boolean(),
          ease4: z.number(),
          fuzz: z.number(),
          ivlFct: z.number(),
          maxIvl: z.number(),
          minSpace: z.number(),
          perDay: z.number()
        })
      }).describe("Complete deck configuration object to save")
    },
    async ({ config }) => {
      try {
        const result = await ankiClient.deck.saveDeckConfig({ config });
        if (!result) {
          throw new Error("Failed to save deck configuration - operation returned false");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully saved deck configuration "${config.name}" (ID: ${config.id})`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to save deck configuration: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "set_deck_config",
    {
      configId: z.number().describe("ID of the configuration to apply"),
      deckNames: z.array(z.string()).describe("Array of deck names to apply the configuration to")
    },
    async ({ configId, deckNames }) => {
      try {
        const result = await ankiClient.deck.setDeckConfigId({
          configId,
          decks: deckNames
        });
        if (!result) {
          throw new Error("Failed to set deck configuration - operation returned false");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully applied configuration ${configId} to decks: ${deckNames.join(", ")}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set deck configuration: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/tools/cards.ts
import { z as z2 } from "zod";
function registerCardTools(server2) {
  server2.tool(
    "answer_cards",
    {
      answers: z2.array(
        z2.object({
          cardId: z2.number().describe("ID of the card to answer"),
          ease: z2.number().min(1).max(4).describe("Ease rating: 1 (Again), 2 (Hard), 3 (Good), 4 (Easy)")
        })
      ).describe("Array of card answers")
    },
    async ({ answers }) => {
      try {
        const results = await ankiClient.card.answerCards({ answers });
        const successCount = results.filter(Boolean).length;
        const failureCount = results.length - successCount;
        return {
          content: [
            {
              type: "text",
              text: `Answered ${successCount} cards successfully, ${failureCount} failed. Results: ${JSON.stringify(results)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to answer cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "find_cards",
    {
      query: z2.string().describe("Anki search query to find cards")
    },
    async ({ query }) => {
      try {
        const cardIds = await ankiClient.card.findCards({ query });
        return {
          content: [
            {
              type: "text",
              text: `Found ${cardIds.length} cards matching query "${query}": ${JSON.stringify(cardIds)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to find cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_cards_info",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get information for")
    },
    async ({ cardIds }) => {
      try {
        const cardsInfo = await ankiClient.card.cardsInfo({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Card information: ${JSON.stringify(cardsInfo, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get cards info: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "check_cards_due",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to check")
    },
    async ({ cardIds }) => {
      try {
        const areDue = await ankiClient.card.areDue({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          isDue: areDue[index]
        }));
        const dueCount = areDue.filter(Boolean).length;
        return {
          content: [
            {
              type: "text",
              text: `${dueCount} out of ${cardIds.length} cards are due. Details: ${JSON.stringify(result)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if cards are due: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "check_cards_suspended",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to check")
    },
    async ({ cardIds }) => {
      try {
        const areSuspended = await ankiClient.card.areSuspended({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          isSuspended: areSuspended[index]
        }));
        const suspendedCount = areSuspended.filter((status) => status === true).length;
        return {
          content: [
            {
              type: "text",
              text: `${suspendedCount} out of ${cardIds.length} cards are suspended. Details: ${JSON.stringify(result)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if cards are suspended: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "suspend_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to suspend")
    },
    async ({ cardIds }) => {
      try {
        const result = await ankiClient.card.suspend({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully suspended ${cardIds.length} cards. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to suspend cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "unsuspend_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to unsuspend")
    },
    async ({ cardIds }) => {
      try {
        const result = await ankiClient.card.unsuspend({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully unsuspended ${cardIds.length} cards. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to unsuspend cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "check_card_suspended",
    {
      cardId: z2.number().describe("Card ID to check")
    },
    async ({ cardId }) => {
      try {
        const isSuspended = await ankiClient.card.suspended({ card: cardId });
        return {
          content: [
            {
              type: "text",
              text: `Card ${cardId} is ${isSuspended ? "suspended" : "not suspended"}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if card is suspended: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "forget_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to forget")
    },
    async ({ cardIds }) => {
      try {
        await ankiClient.card.forgetCards({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully forgot ${cardIds.length} cards, making them new again`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to forget cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "relearn_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to relearn")
    },
    async ({ cardIds }) => {
      try {
        await ankiClient.card.relearnCards({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully set ${cardIds.length} cards to relearn`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to relearn cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "set_cards_due_date",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to set due date for"),
      days: z2.string().describe("Number of days from today (can be negative for past dates)")
    },
    async ({ cardIds, days }) => {
      try {
        const result = await ankiClient.card.setDueDate({ cards: cardIds, days });
        return {
          content: [
            {
              type: "text",
              text: `Successfully set due date for cards. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set due date for cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_cards_ease_factors",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get ease factors for")
    },
    async ({ cardIds }) => {
      try {
        const easeFactors = await ankiClient.card.getEaseFactors({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          easeFactor: easeFactors[index]
        }));
        return {
          content: [
            {
              type: "text",
              text: `Ease factors: ${JSON.stringify(result, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get ease factors: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "set_cards_ease_factors",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to set ease factors for"),
      easeFactors: z2.array(z2.number()).describe("Array of ease factors (must match the length of cardIds)")
    },
    async ({ cardIds, easeFactors }) => {
      try {
        if (cardIds.length !== easeFactors.length) {
          throw new Error("Number of card IDs must match number of ease factors");
        }
        const results = await ankiClient.card.setEaseFactors({
          cards: cardIds,
          easeFactors
        });
        const successCount = results.filter(Boolean).length;
        return {
          content: [
            {
              type: "text",
              text: `Successfully set ease factors for ${successCount} out of ${cardIds.length} cards`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set ease factors: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_cards_intervals",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get intervals for"),
      complete: z2.boolean().optional().describe("If true, returns all intervals; if false, returns only the most recent")
    },
    async ({ cardIds, complete = false }) => {
      try {
        const intervals = await ankiClient.card.getIntervals({ cards: cardIds, complete });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          intervals: intervals[index]
        }));
        return {
          content: [
            {
              type: "text",
              text: `Intervals (${complete ? "complete history" : "most recent"}): ${JSON.stringify(result, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get intervals: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_cards_mod_time",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get modification time for")
    },
    async ({ cardIds }) => {
      try {
        const modTimes = await ankiClient.card.cardsModTime({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Modification times: ${JSON.stringify(modTimes, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get modification times: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "cards_to_notes",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get note IDs for")
    },
    async ({ cardIds }) => {
      try {
        const noteIds = await ankiClient.card.cardsToNotes({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Note IDs: ${JSON.stringify(noteIds)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get note IDs from cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "set_card_specific_values",
    {
      cardId: z2.number().describe("Card ID to modify"),
      keys: z2.array(
        z2.enum([
          "data",
          "did",
          "due",
          "factor",
          "flags",
          "id",
          "ivl",
          "lapses",
          "left",
          "mod",
          "odid",
          "odue",
          "ord",
          "queue",
          "reps",
          "type",
          "usn"
        ])
      ).describe("Array of card property keys to modify"),
      newValues: z2.array(z2.string()).describe("Array of new values (must match the length of keys)")
    },
    async ({ cardId, keys, newValues }) => {
      try {
        if (keys.length !== newValues.length) {
          throw new Error("Number of keys must match number of new values");
        }
        const results = await ankiClient.card.setSpecificValueOfCard({
          card: cardId,
          keys,
          newValues
        });
        const successCount = results.filter(Boolean).length;
        return {
          content: [
            {
              type: "text",
              text: `Successfully updated ${successCount} out of ${keys.length} values for card ${cardId}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set card values: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/tools/notes.ts
import { z as z3 } from "zod";
function registerNoteTools(server2) {
  server2.tool(
    "add_note",
    {
      deckName: z3.string().describe("Name of the deck to add the note to"),
      modelName: z3.string().describe('Name of the note model/type (e.g., "Basic", "Cloze")'),
      fields: z3.record(z3.string()).describe("Object with field names as keys and field content as values"),
      tags: z3.array(z3.string()).optional().describe("Array of tags to add to the note")
    },
    async ({ deckName, modelName, fields, tags }) => {
      try {
        const note = {
          deckName,
          modelName,
          fields,
          tags: tags || []
        };
        const noteId = await ankiClient.note.addNote({ note });
        if (noteId === null) {
          throw new Error("Failed to add note - possibly a duplicate or invalid fields");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully added note with ID: ${noteId}. Deck: "${deckName}", Model: "${modelName}", Tags: [${tags?.join(", ") || "none"}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to add note: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "add_notes",
    {
      notes: z3.array(z3.object({
        deckName: z3.string().describe("Name of the deck to add the note to"),
        modelName: z3.string().describe("Name of the note model/type"),
        fields: z3.record(z3.string()).describe("Object with field names as keys and field content as values"),
        tags: z3.array(z3.string()).optional().describe("Array of tags to add to the note")
      })).describe("Array of notes to add")
    },
    async ({ notes }) => {
      try {
        const formattedNotes = notes.map((note) => ({
          ...note,
          tags: note.tags || []
        }));
        const results = await ankiClient.note.addNotes({ notes: formattedNotes });
        const successCount = results?.filter((result) => result !== null).length || 0;
        const failureCount = (results?.length || 0) - successCount;
        return {
          content: [
            {
              type: "text",
              text: `Batch note addition completed. Successfully added: ${successCount} notes, Failed: ${failureCount} notes. Results: ${JSON.stringify(results)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to add notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "update_note",
    {
      noteId: z3.number().describe("ID of the note to update"),
      fields: z3.record(z3.string()).optional().describe("Object with field names as keys and new field content as values"),
      tags: z3.array(z3.string()).optional().describe("Array of tags to set for the note")
    },
    async ({ noteId, fields, tags }) => {
      try {
        if (!fields && !tags) {
          throw new Error("Either fields or tags must be provided for update");
        }
        const updateData = { id: noteId };
        if (fields) updateData.fields = fields;
        if (tags) updateData.tags = tags;
        await ankiClient.note.updateNote({ note: updateData });
        return {
          content: [
            {
              type: "text",
              text: `Successfully updated note ${noteId}. ${fields ? "Updated fields. " : ""}${tags ? `Updated tags: [${tags.join(", ")}]` : ""}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to update note ${noteId}: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "delete_notes",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to delete")
    },
    async ({ noteIds }) => {
      try {
        await ankiClient.note.deleteNotes({ notes: noteIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully deleted ${noteIds.length} notes: [${noteIds.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to delete notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "add_tags_to_notes",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to add tags to"),
      tags: z3.string().describe("Space-separated string of tags to add")
    },
    async ({ noteIds, tags }) => {
      try {
        await ankiClient.note.addTags({ notes: noteIds, tags });
        return {
          content: [
            {
              type: "text",
              text: `Successfully added tags "${tags}" to ${noteIds.length} notes: [${noteIds.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to add tags to notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "remove_tags_from_notes",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to remove tags from"),
      tags: z3.string().describe("Space-separated string of tags to remove")
    },
    async ({ noteIds, tags }) => {
      try {
        await ankiClient.note.removeTags({ notes: noteIds, tags });
        return {
          content: [
            {
              type: "text",
              text: `Successfully removed tags "${tags}" from ${noteIds.length} notes: [${noteIds.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to remove tags from notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "replace_tags_in_notes",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to replace tags in"),
      tagToReplace: z3.string().describe("Tag to replace"),
      replaceWithTag: z3.string().describe("Tag to replace with")
    },
    async ({ noteIds, tagToReplace, replaceWithTag }) => {
      try {
        await ankiClient.note.replaceTags({
          notes: noteIds,
          tag_to_replace: tagToReplace,
          replace_with_tag: replaceWithTag
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully replaced tag "${tagToReplace}" with "${replaceWithTag}" in ${noteIds.length} notes: [${noteIds.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to replace tags in notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "find_notes",
    {
      query: z3.string().describe("Anki search query to find notes"),
      includeInfo: z3.boolean().default(false).describe("Whether to include detailed note information"),
      limit: z3.number().default(50).describe("Maximum number of notes to return detailed info for")
    },
    async ({ query, includeInfo, limit }) => {
      try {
        const noteIds = await ankiClient.note.findNotes({ query });
        let result = `Found ${noteIds.length} notes matching query: "${query}"`;
        if (includeInfo && noteIds.length > 0) {
          const limitedIds = noteIds.slice(0, limit);
          const notesInfo = await ankiClient.note.notesInfo({ notes: limitedIds });
          result += `

Detailed information for first ${limitedIds.length} notes:
${JSON.stringify(notesInfo, null, 2)}`;
        } else {
          result += `

Note IDs: [${noteIds.join(", ")}]`;
        }
        return {
          content: [
            {
              type: "text",
              text: result
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to find notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "can_add_notes",
    {
      notes: z3.array(z3.object({
        deckName: z3.string().describe("Name of the deck to add the note to"),
        modelName: z3.string().describe("Name of the note model/type"),
        fields: z3.record(z3.string()).describe("Object with field names as keys and field content as values"),
        tags: z3.array(z3.string()).optional().describe("Array of tags to add to the note")
      })).describe("Array of notes to validate")
    },
    async ({ notes }) => {
      try {
        const formattedNotes = notes.map((note) => ({
          ...note,
          tags: note.tags || []
        }));
        const canAddResults = await ankiClient.note.canAddNotes({ notes: formattedNotes });
        const validCount = canAddResults.filter((result) => result).length;
        const invalidCount = canAddResults.length - validCount;
        return {
          content: [
            {
              type: "text",
              text: `Note validation completed. Valid: ${validCount}, Invalid: ${invalidCount}. Results: ${JSON.stringify(canAddResults)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to validate notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "clear_unused_tags",
    {},
    async () => {
      try {
        const removedTags = await ankiClient.note.clearUnusedTags();
        return {
          content: [
            {
              type: "text",
              text: `Successfully cleared ${removedTags.length} unused tags: [${removedTags.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to clear unused tags: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "remove_empty_notes",
    {},
    async () => {
      try {
        await ankiClient.note.removeEmptyNotes();
        return {
          content: [
            {
              type: "text",
              text: "Successfully removed all empty notes from the collection"
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to remove empty notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/index.ts
var server = new McpServer4({
  name: "Anki MCP Server",
  version: "0.1.0"
});
registerDeckResources(server);
registerDeckTools(server);
registerCardResources(server);
registerCardTools(server);
registerNoteResources(server);
registerNoteTools(server);
var transport = new StdioServerTransport();
await server.connect(transport);
//# sourceMappingURL=index.js.map